import click

from taxonomical_utils.merger import merge_files
from taxonomical_utils.resolver import resolve_taxa
from taxonomical_utils.upper_taxa_lineage_appender import append_upper_taxa_lineage


@click.group()
def cli() -> None:
    pass


@click.command()
@click.option("--input-file", required=True, type=click.Path(exists=True), help="Path to the input file.")
@click.option("--output-file", required=True, type=click.Path(), help="Path to the output file.")
@click.option("--org-column-header", required=True, type=str, help="Column header for the organism.")
def resolve(input_file: str, output_file: str, org_column_header: str) -> None:
    resolve_taxa(input_file, output_file, org_column_header)


@click.command()
@click.option("--input-file", required=True, type=click.Path(exists=True), help="Path to the input file.")
@click.option("--output-file", required=True, type=click.Path(), help="Path to the output file.")
def append_taxonomy(input_file: str, output_file: str) -> None:
    append_upper_taxa_lineage(input_file, output_file)


@click.command()
@click.option("--input-file", required=True, type=click.Path(exists=True), help="Path to the input file.")
@click.option(
    "--resolved-taxa-file", required=True, type=click.Path(exists=True), help="Path to the resolved taxa file."
)
@click.option(
    "--upper-taxa-lineage-file",
    required=True,
    type=click.Path(exists=True),
    help="Path to the upper taxa lineage file.",
)
@click.option("--output-file", required=True, type=click.Path(), help="Path to the output file.")
@click.option("--org-column-header", required=True, type=str, help="Column header for the organism.")
@click.option("--delimiter", default=",", type=str, help="Delimiter of the input file.")
def merge(
    input_file: str,
    resolved_taxa_file: str,
    upper_taxa_lineage_file: str,
    output_file: str,
    org_column_header: str,
    delimiter: str,
) -> None:
    merge_files(input_file, resolved_taxa_file, upper_taxa_lineage_file, output_file, org_column_header, delimiter)


cli.add_command(resolve)
cli.add_command(append_taxonomy)
cli.add_command(merge)

if __name__ == "__main__":
    cli()



import requests


def switch_downloader(switch_id: str, path_to_file: str, timeout: int = 30) -> None:
    """Downloads a switch file if given a switch id
    Args:
        switch_id (string): a SWITCH Drive id
        path_to_file (string): local path to save the downloaded file
        timeout (int): timeout in seconds for the request
    Returns:
        nothing
    """
    switch_url = f"https://drive.switch.ch/index.php/s/{switch_id}/download"
    response = requests.get(switch_url, timeout=timeout)
    response.raise_for_status()  # Check if the request was successful

    with open(path_to_file, "wb") as file:
        file.write(response.content)



from typing import Any, Dict, List

import pandas as pd
from opentree import OT

path_to_input_file = "data/in/example.csv"
org_column_header = "idTaxon"
delimiter = ","


def process_species_list(
    path_to_input_file: str, org_column_header: str = "source_taxon", delimiter: str = ","
) -> pd.DataFrame:
    species_list_df = pd.read_csv(path_to_input_file, sep=delimiter, encoding="utf-8")
    species_list_df.columns = species_list_df.columns.str.strip()  # Strip any leading/trailing whitespace
    # First, we copy the original column to a new column with a standardized name

    species_list_df["taxon_search_string"] = species_list_df[org_column_header]
    species_list_df["taxon_search_string"].dropna(inplace=True)
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.lower()
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.replace(r" sp ", "", regex=True)
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.replace(r" x ", " ", regex=True)
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.replace(r" × ", " ", regex=True)  # noqa: RUF001
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.replace(r" x$", "", regex=True)
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.replace(r" ×$", "", regex=True)  # noqa: RUF001

    # List of invalid or placeholder taxon strings
    invalid_taxon_strings = ["nan", "nd", "", None]

    # Filter out rows with invalid taxon strings
    species_list_df["taxon_search_string"] = species_list_df[
        ~species_list_df["taxon_search_string"].isin(invalid_taxon_strings)
    ]["taxon_search_string"]

    # Here the first two words are taken as the genus and species
    species_list_df["taxon_search_string"] = species_list_df["taxon_search_string"].str.split().str[:2].str.join(" ")
    return species_list_df


def resolve_organisms(organisms: List[str]) -> Dict[str, Any]:
    results: Dict[str, Any] = {"results": [], "unmatched_names": []}
    for organism in organisms:
        match = OT.tnrs_match([organism], context_name=None, do_approximate_matching=True, include_suppressed=False)
        results["results"].extend(match.response_dict["results"])
        results["unmatched_names"].extend(match.response_dict["unmatched_names"])
    return results



import pandas as pd

from taxonomical_utils.processor import process_species_list


def merge_files(
    input_file: str,
    resolved_taxa_file: str,
    upper_taxa_lineage_file: str,
    output_file: str,
    org_column_header: str,
    delimiter: str = ",",
) -> pd.DataFrame:
    # Process the species list in the input file
    processed_species_list_df = process_species_list(input_file, org_column_header, delimiter)

    # Load the resolved taxa file and upper taxa lineage file
    resolved_taxa_df = pd.read_csv(resolved_taxa_file)
    upper_taxa_lineage_df = pd.read_csv(upper_taxa_lineage_file)

    # Merge the resolved taxa file and upper taxa lineage file on 'taxon.ott_id' and 'ott_id'
    merged_taxa_df = pd.merge(
        resolved_taxa_df, upper_taxa_lineage_df, left_on="taxon.ott_id", right_on="ott_id", how="left"
    )

    # Merge the input dataframe with the merged taxa dataframe
    final_df = pd.merge(
        processed_species_list_df,
        merged_taxa_df,
        left_on="taxon_search_string",
        right_on="search_string",
        how="left",
        suffixes=("_input", "_merged"),
    )

    # Save the final dataframe to the output file
    final_df.to_csv(output_file, sep=",", index=False, encoding="utf-8")

    return final_df



import os

import pandas as pd

from taxonomical_utils.exceptions import FileDownloadError
from taxonomical_utils.processor import process_species_list, resolve_organisms
from taxonomical_utils.shared import load_json, normalize_json_resolver, save_json


def resolve_taxa(
    input_file: str,
    output_file: str,
    org_column_header: str,
) -> pd.DataFrame:
    # Define paths
    path_to_input_file = input_file
    organisms_tnrs_matched_filename = f"{os.path.splitext(input_file)[0]}_organisms.json"

    # Check if the input file exists
    if not os.path.isfile(path_to_input_file):
        raise FileDownloadError(path_to_file=path_to_input_file)

    # Detect file type by extension if delimiter is not specified
    delimiter = "," if input_file.endswith(".csv") else "\t"

    # Process species list
    species_list_df = process_species_list(path_to_input_file, org_column_header, delimiter)

    # Resolve organisms
    organisms = species_list_df["taxon_search_string"].unique().tolist()
    # Here also we make sure to remove NaN values
    organisms = [x for x in organisms if str(x) != "nan"]
    print(f"Resolving {len(organisms)} organisms")
    print(organisms)
    organisms_tnrs_matched = resolve_organisms(organisms)

    # Save matched organisms to json
    save_json(organisms_tnrs_matched, organisms_tnrs_matched_filename)

    # Load and normalize json
    json_data = load_json(organisms_tnrs_matched_filename)
    df_organism_tnrs_matched, df_organism_tnrs_unmatched = normalize_json_resolver(json_data)

    # Process the results and update the dataframe
    df_organism_tnrs_matched.sort_values(["search_string", "is_synonym"], axis=0, inplace=True)

    # Ensure we keep all unique matches for each search_string
    df_organism_tnrs_matched_unique = df_organism_tnrs_matched.drop_duplicates(
        subset=["search_string"], keep="first"
    ).copy()

    # Drop duplicates based on the provided org_column_header
    df_organism_tnrs_matched_unique.drop_duplicates(
        subset=["search_string", "matched_name", "taxon.ott_id"], keep="first", inplace=True
    )

    # Save the final dataframe
    df_organism_tnrs_matched_unique.to_csv(output_file, sep=",", index=False, encoding="utf-8")

    return df_organism_tnrs_matched_unique
